package be.iminds.ilabt.jfed.restapi.json;

import be.iminds.ilabt.jfed.util.GeniUrn;
import be.iminds.ilabt.util.jsonld.JsonLdObjectsMetaData;
import be.iminds.ilabt.util.jsonld.impl.PrimaryIdObjectBuilder;

import java.net.URI;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

@JsonLdObjectsMetaData.JsonLdObjectBuilderInfo(objectClass = User.class)
public class UserBuilder extends PrimaryIdObjectBuilder<String, User, UserBuilder> {
    private GeniUrn urn;
    private String authorityTld;
    private URL authorityServer;

    private List<Slice> slices;
    private List<Project> projects;

    public UserBuilder() {
    }

    public UserBuilder(User orig) {
        super(orig);
        this.urn = orig.getUrn();
        this.authorityTld = orig.getAuthorityTld();
        this.authorityServer = orig.getAuthorityServer();
        this.projects = orig.getProjects() == null ? null : new ArrayList<>(orig.getProjects());
        this.slices = orig.getSlices() == null ? null : new ArrayList<>(orig.getSlices());
    }

    public UserBuilder setUrn(GeniUrn urn) {
        this.urn = urn;
        return this;
    }

    public UserBuilder setAuthorityTld(String authorityTld) {
        this.authorityTld = authorityTld;
        return this;
    }

    public UserBuilder setAuthorityServer(URL authorityServer) {
        this.authorityServer = authorityServer;
        return this;
    }

    public UserBuilder setSlices(List<Slice> slices) {
        this.slices = slices;
        return this;
    }

    public UserBuilder addSlice(Slice slice) {
        if (this.slices == null) {
            this.slices = new ArrayList<>();
        }

        this.slices.add(slice);
        return this;
    }

    public UserBuilder addSlice(GeniUrn slice) {
        addSlice(new Slice(slice.getResourceName(), slice.getSubAuthName(), slice, null/*experimentUrl*/));
        return this;
    }

    public UserBuilder addSlice(GeniUrn slice, URL experimentUrl) {
        addSlice(new Slice(slice.getResourceName(), slice.getSubAuthName(), slice, experimentUrl));
        return this;
    }

    public UserBuilder setProjects(List<Project> projects) {
        this.projects = projects;
        return this;
    }

    public UserBuilder addProject(Project project) {
        if (this.projects == null) {
            this.projects = new ArrayList<>();
        }

        this.projects.add(project);
        return this;
    }

    public UserBuilder addProject(GeniUrn project) {
        addProject(new Project(project.getResourceName(), project));
        return this;
    }


    public User create() {
        return new User(id, urn, authorityTld, authorityServer, slices, projects, uri,
                mustSerializeAsEmbeddedObjectForNonMinimizedCreate());
    }

    @Override
    public User createMinimized(JsonLdObjectsMetaData.Minimization minimization) {
        return new User(
                minimization.includeId() ? id : null,
                minimization.includeOtherProperties() ? urn : null,
                minimization.includeOtherProperties() ? authorityTld : null,
                minimization.includeOtherProperties() ? authorityServer : null,
                minimization.includeOtherProperties() ? slices : null,
                minimization.includeOtherProperties() ? projects : null,
                minimization.includeId() ? uri : null,
                minimization.serializeAsEmbeddedObject());
    }

    public GeniUrn getUrn() { return urn; }
    public String getAuthorityTld() { return authorityTld; }
    public URL getAuthorityServer() { return authorityServer; }
    public List<Slice> getSlices() { return slices; }
    public List<Project> getProjects() { return projects; }
}