package be.iminds.ilabt.jfed.restapi.json;


import be.iminds.ilabt.jfed.util.GeniUrn;
import be.iminds.ilabt.jfed.util.RFC3339Util;
import be.iminds.ilabt.util.jsonld.test.ObjectWithIdTestCollection;

import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.equalTo;
import static org.hamcrest.Matchers.is;
import static org.hamcrest.Matchers.notNullValue;

public class ExperimentTestCollection extends ObjectWithIdTestCollection<Long, Experiment, ExperimentBuilder> {
    public ExperimentTestCollection() throws Exception {
        super(Long.class, Experiment.class, ExperimentBuilder.class);

        ExperimentBuilder exp1 = new ExperimentBuilder();
//        exp1.setName("myExpName");
        exp1.setId(1548l);

        exp1.setStatus(Experiment.Status.STARTING);
        exp1.setGlobalSliverStatus("CHANGING");
        exp1.setAllocationStatus("geni_provisioned");
        exp1.setOperationalStatus("geni_configuring");
        exp1.setJFedExperimentState("PROVISIONING");
//        exp1.setStatusDetails(new Experiment.StatusDetails(
//                null,
//                "geni_provisioned",
//                "geni_configuring",
//                "PROVISIONING"));

        exp1.setAm(new URL("http://fls.fed4fire.eu/api2/server/320"));
        exp1.setCreated(new Timestamp(RFC3339Util.rfc3339StringToDate("2015-11-30T10:00:00Z").getTime()));
        exp1.setExpire(new Timestamp(RFC3339Util.rfc3339StringToDate("2015-11-30T16:00:00Z").getTime()));
        exp1.setManifestRspec("<rspec type=\"manifest\"...>");
        exp1.setRequestRspec("<rspec type=\"request\"...>");
        exp1.setOwner(new GeniUrn("urn:publicid:IDN+wall2.ilabt.iminds.be:test+user+user1"));
        exp1.addSharedWith(new GeniUrn("urn:publicid:IDN+wall2.ilabt.iminds.be:test+user+user2"));
        exp1.addSharedWith(new GeniUrn("urn:publicid:IDN+testing.jfed.iminds.be+user+tester1"));
        exp1.setProject("test");
        exp1.setSliceName("slice1");
        exp1.setSliceUrn(new GeniUrn("urn:publicid:IDN+wall2.ilabt.iminds.be:test+slice+slice1"));
        exp1.setSpeakingFor("urn:publicid:IDN+example.com+user+spokenForUsers");
        exp1.setSpeaksForCredential("<SPEAKSFOR CREDENTIAL XML HERE>");

        all.add(exp1);



        ExperimentBuilder exp2 = new ExperimentBuilder();
        exp2.setId(1444l);

        exp2.setStatus(Experiment.Status.READY);
        exp2.setAllocationStatus("geni_provisioned");
        exp2.setOperationalStatus("geni_ready");
        exp2.setJFedExperimentState("READY");
        exp2.setGlobalSliverStatus("TEST");
        exp2.setAm(new URL("http://fls.fed4fire.eu/api2/server/321"));
        exp2.setCreated(new Timestamp(RFC3339Util.rfc3339StringToDate("2016-11-30T10:00:00Z").getTime()));
        exp2.setExpire(new Timestamp(RFC3339Util.rfc3339StringToDate("2016-11-30T16:00:00Z").getTime()));
        exp2.setManifestRspec("<rspec type=\"manifest\" 2 ...>");
        exp2.setRequestRspec("<rspec type=\"request\" 2 ...>");
        exp2.setOwner(new GeniUrn("urn:publicid:IDN+wall2.ilabt.iminds.be:test+user+user2"));
        exp2.addSharedWith(new GeniUrn("urn:publicid:IDN+wall2.ilabt.iminds.be:test+user+user1"));
        exp2.setProject("testB");
        exp2.setSliceName("slice2");
        exp2.setSliceUrn(new GeniUrn("urn:publicid:IDN+wall2.ilabt.iminds.be:test+slice+slice2"));
        exp2.setSpeakingFor("urn:publicid:IDN+example.com+user+spokenForUser2");
        exp2.setSpeaksForCredential("<SPEAKSFOR CREDENTIAL XML HERE 2>");

        all.add(exp2);

        ExperimentBuilder exp3 = new ExperimentBuilder();
        exp3.setId(1555l);

        exp3.setStatus(Experiment.Status.READY);
        exp3.setAllocationStatus("geni_provisioned");
        exp3.setOperationalStatus("geni_ready");
        exp3.setJFedExperimentState("READY");
        exp3.setGlobalSliverStatus("TEST");
        exp3.setAm(null);
        exp3.setCreated(new Timestamp(RFC3339Util.rfc3339StringToDate("2016-11-30T10:00:00Z").getTime()));
        exp3.setExpire(new Timestamp(RFC3339Util.rfc3339StringToDate("2016-11-30T16:00:00Z").getTime()));
        exp3.setManifestRspec(null);
        exp3.setRequestRspec(null);
        exp3.setOwner(new GeniUrn("urn:publicid:IDN+wall2.ilabt.iminds.be:test+user+user2"));
        exp3.setProject("testB");
        exp3.setSliceName("slice2");
        exp3.setSliceUrn(new GeniUrn("urn:publicid:IDN+wall2.ilabt.iminds.be:test+slice+slice2"));
        exp3.setSpeakingFor("urn:publicid:IDN+example.com+user+spokenForUser2");
        exp3.setSpeaksForCredential(null);

        all.add(exp3);
    }

//    public String getFixtureByIndex(int index) {
//        return "fixtures/experiment-"+index+".json";
//    }

    public Experiment getBySliceUrn(GeniUrn sliceUrn) {
        for (ExperimentBuilder builder : all) {
            if (builder.getSliceUrn() != null && builder.getSliceUrn().equals(sliceUrn))
                return builder.create();
        }
        return null;
    }
    public Experiment getBySliceName(String sliceName) {
        for (ExperimentBuilder builder : all) {
            if (builder.getSliceName() != null && builder.getSliceName().equals(sliceName))
                return builder.create();
        }
        return null;
    }

    @Override
    public void assertSameExtraIds(Experiment experiment, Experiment t1) {

    }

    @Override
    public void assertSameDetails(Experiment actual, Experiment expected) {
        assertThat("am differs", actual.getAm(), is(equalTo(expected.getAm())));
        assertThat("created differs", actual.getCreated(), is(equalTo(expected.getCreated())));
        assertThat("expires differs", actual.getExpire(), is(equalTo(expected.getExpire())));
        assertThat("id differs", actual.getId(), is(equalTo(expected.getId())));
        assertThat("manifest differs", actual.getManifestRspec(), is(equalTo(expected.getManifestRspec())));
        assertThat(actual.getOwner(), is(equalTo(expected.getOwner())));
        assertThat(actual.getProject(), is(equalTo(expected.getProject())));
        assertThat("requestRspec differs", actual.getRequestRspec(), is(equalTo(expected.getRequestRspec())));
        assertThat("sharedWith differs", actual.getSharedWith(), is(equalTo(expected.getSharedWith())));
        assertThat(actual.getSliceName(), is(equalTo(expected.getSliceName())));
        assertThat(actual.getSliceUrn(), is(equalTo(expected.getSliceUrn())));
        assertThat(actual.getSpeakingFor(), is(equalTo(expected.getSpeakingFor())));
        assertThat(actual.getSpeaksForCredential(), is(equalTo(expected.getSpeaksForCredential())));
        assertThat(actual.getStatus(), is(equalTo(expected.getStatus())));
        assertThat(actual.getStatusDetails(), is(equalTo(expected.getStatusDetails())));
    }

    @Override
    public void assertSearchLinks(Experiment experiment) {

    }
}
